'use strict';

function cssHasPseudo(document) {
  const observedItems = [];
  const attributeElement = document.createElement('x'); // walk all stylesheets to collect observed css rules

  Array.prototype.forEach.call(document.styleSheets, walkStyleSheet);
  transformObservedItems(); // observe DOM modifications that affect selectors

  const mutationObserver = new MutationObserver(mutationsList => {
    mutationsList.forEach(mutation => {
      Array.prototype.forEach.call(mutation.addedNodes || [], node => {
        // walk stylesheets to collect observed css rules
        if (node.nodeType === 1 && node.sheet) {
          walkStyleSheet(node.sheet);
        }
      }); // transform observed css rules

      cleanupObservedCssRules();
      transformObservedItems();
    });
  });
  mutationObserver.observe(document, {
    childList: true,
    subtree: true
  }); // observe DOM events that affect pseudo-selectors

  document.addEventListener('focus', () => setImmediate(transformObservedItems), true);
  document.addEventListener('blur', () => setImmediate(transformObservedItems), true);
  document.addEventListener('input', () => setImmediate(transformObservedItems)); // transform observed css rules

  function transformObservedItems() {
    observedItems.forEach(item => {
      const nodes = [];
      Array.prototype.forEach.call(document.querySelectorAll(item.scopeSelector), element => {
        const nthChild = Array.prototype.indexOf.call(element.parentNode.children, element) + 1;
        const relativeSelectors = item.relativeSelectors.map(relativeSelector => item.scopeSelector + ':nth-child(' + nthChild + ') ' + relativeSelector).join(); // find any relative :has element from the :scope element

        const relativeElement = element.parentNode.querySelector(relativeSelectors);
        const shouldElementMatch = item.isNot ? !relativeElement : relativeElement;

        if (shouldElementMatch) {
          // memorize the node
          nodes.push(element); // set an attribute with an irregular attribute name

          attributeElement.innerHTML = '<x ' + item.attributeName + '>';
          element.attributes.setNamedItem(attributeElement.firstChild.attributes[0].cloneNode()); // trigger a style refresh in IE and Edge

          document.documentElement.style.zoom = 1;
          document.documentElement.style.zoom = null;
        }
      }); // remove the encoded attribute from all nodes that no longer match them

      item.nodes.forEach(node => {
        if (nodes.indexOf(node) === -1) {
          node.removeAttribute(item.attributeName); // trigger a style refresh in IE and Edge

          document.documentElement.style.zoom = 1;
          document.documentElement.style.zoom = null;
        }
      }); // update the

      item.nodes = nodes;
    });
  } // remove any observed cssrules that no longer apply


  function cleanupObservedCssRules() {
    Array.prototype.push.apply(observedItems, observedItems.splice(0).filter(item => item.rule.parentStyleSheet && item.rule.parentStyleSheet.ownerNode && document.documentElement.contains(item.rule.parentStyleSheet.ownerNode)));
  } // walk a stylesheet to collect observed css rules


  function walkStyleSheet(styleSheet) {
    // walk a css rule to collect observed css rules
    Array.prototype.forEach.call(styleSheet.cssRules, rule => {
      // decode the selector text in all browsers to:
      // [1] = :scope, [2] = :not(:has), [3] = :has relative, [4] = :scope relative
      const selectors = decodeURIComponent(rule.selectorText.replace(/\\(.)/g, '$1')).match(/^(.*?)\[:(not-)?has\((.+?)\)\](.*?)$/);

      if (selectors) {
        const attributeName = ':' + (selectors[2] ? 'not-' : '') + 'has(' + // encode a :has() pseudo selector as an attribute name
        selectors[3].replace(/%3A/g, ':').replace(/%5B/g, '[').replace(/%5D/g, ']').replace(/%2C/g, ',') + ')';
        observedItems.push({
          rule,
          scopeSelector: selectors[1],
          isNot: selectors[2],
          relativeSelectors: selectors[3].split(/\s*,\s*/),
          attributeName,
          nodes: []
        });
      }
    });
  }
}

module.exports = cssHasPseudo;
//# sourceMappingURL=index.js.map
